/* Copyright 2015 Thomas Flynn */
/*This file is part of asc.

    asc is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    asc is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with asc.  If not, see <http://www.gnu.org/licenses/>.*/

package com.androidscheme.tom.myapplication;

import android.os.Environment;
import android.widget.EditText;
import android.widget.TextView;


import java.io.File;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Set;
import java.util.concurrent.BlockingQueue;
import org.tomthebomb.ajl.Env;
import org.tomthebomb.ajl.fn_vals;
import org.tomthebomb.ajl.lisp;
import org.tomthebomb.ajl.sym_num;
import org.tomthebomb.ajl.sym_vals;

/**
 * Created by tom on 7/19/15.
 */
class androidlisp implements Runnable{

    public static Env android_env(TextView runText, BlockingQueue<String> inputQueue,FullscreenActivity act){
        //Make a std environment
        Env ret = lisp.core_env();
        ret.put("input",new androidinput(inputQueue));
        ret.put("display",new androiddisplay(runText,act));
        ret.put("print",new androidprint(runText,act));
        ret.put("load", new androidload(act));
        ret.put("newline",new androidnewline(runText,act));

     //   System.out.println("Bound androidprint NEW");
        return ret;
    }

    public void run() {
        try {
            lisp.runProg_throw(prog, env);
        }
        catch (Throwable e) {
            if(Thread.currentThread().isInterrupted()){
                System.out.println("Interrupted!");
                return;
            }
            act.anstr = e.toString();//will print the exception
            act.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    FullscreenActivity.addLine((TextView) act.findViewById(com.androidscheme.tom.myapplication.R.id.runView),
                            act.anstr);
                }
            });
        }

        System.out.println("Exec'd prog");
    }

    public void setProg(String _prog) {
            prog = _prog;
        }

    public void run(String str) {
        try {
            lisp.runProg_throw(str, env);
        } catch (Throwable e) {
           act.anstr = e.toString();
            act.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    act.addLine((TextView) act.findViewById(com.androidscheme.tom.myapplication.R.id.runView),
                            act.anstr);
                }
            });
        }


    }

    public androidlisp(TextView runText, BlockingQueue<String> inputQueue, FullscreenActivity _act) {
        act = _act;
        env = androidlisp.android_env(runText, inputQueue, act); //environment with proper input & print bindings\
        // System.out.println("Allocated android env");
    }

    Env env;
    String prog;
    FullscreenActivity act;

}

class androidload extends fn_vals {

    public androidload(FullscreenActivity _act){
        act = _act;
    }
    public sym_vals fn_val(LinkedList<sym_vals> args, Env env){
        File newFolder = new File(Environment.getExternalStorageDirectory(),"asc_code");
        if(!newFolder.exists()) {
            System.out.println("Does not exist!");
            newFolder.mkdir();
        }
        String codename = args.getFirst().str_val();

        File f = new File(newFolder,codename);
        if(!files_in_progress.contains(f)) {
            System.out.println("OK to load " + f.toString());
            files_in_progress.add(f);

            act.lisprun.run(localio.read_contents(f));
            files_in_progress.remove(f);
        }
        else{
            System.out.println("Will not load " + f.toString());
        }
        return null;
    }

    FullscreenActivity act;

    static Set<File> files_in_progress = new HashSet<File>(); //holds filenames as they are loaded,
}
class androidinput extends fn_vals {
    public androidinput(BlockingQueue<String> iq){
        inputQueue = iq;
    }

    public sym_vals fn_val(LinkedList<sym_vals> args){
        String st="";
        try {
            st = inputQueue.take();
        }
        catch(InterruptedException e) {
            System.out.println("Caught interrupt");
            Thread.currentThread().interrupt();
            throw new IllegalStateException("Can't complete input");
        }
        catch(Exception e){
            System.out.println("Exception?");
            st = "";
        }
        return new sym_num(
                    Integer.parseInt(st,
                            10));
    }
    BlockingQueue<String> inputQueue;
}

class androidprint extends fn_vals{ //with newline


    public androidprint(TextView rt, FullscreenActivity _act){
        runText=rt; act=_act;
    }


    public sym_vals fn_val(LinkedList<sym_vals> args){
        String str="";
        ListIterator<sym_vals> valsIt = args.listIterator();
        while(valsIt.hasNext()){
            str += valsIt.next().str_val() + " ";
        }
        act.anstr = str;

        BlockingOnUIRunnable actionRunnable = new BlockingOnUIRunnable( act, new BlockingOnUIRunnableListener()
        {
            public void onRunOnUIThread()
            {
                // Execute your activity code here
                runText.setText(
                        runText.getText().toString() + act.anstr + "\n");
                ((EditText)runText).setSelection(runText.getText().length());
            }
        } );

        actionRunnable.startOnUiAndWait();

        return new sym_num(0);
    }
    FullscreenActivity act;
    TextView runText;
}

class androiddisplay extends fn_vals{ //no newline


    public androiddisplay(TextView rt, FullscreenActivity _act){
        runText=rt; act=_act;
    }

    public sym_vals fn_val(LinkedList<sym_vals> args){
        System.out.println("Doing android print");
        String str="";
        ListIterator<sym_vals> valsIt = args.listIterator();
        while(valsIt.hasNext()){
            str += valsIt.next().str_val() + " ";
        }
        act.anstr = str;

        BlockingOnUIRunnable actionRunnable = new BlockingOnUIRunnable( act, new BlockingOnUIRunnableListener()
        {
            public void onRunOnUIThread()
            {
                // Execute your activity code here
                runText.setText(
                        runText.getText().toString() + act.anstr);
                ((EditText)runText).setSelection(runText.getText().length());
            }
        } );

        actionRunnable.startOnUiAndWait();

        return new sym_num(0);
    }
    FullscreenActivity act;
    TextView runText;
}

class androidnewline extends fn_vals{ //no newline

    public androidnewline(TextView rt, FullscreenActivity _act){
        runText=rt; act=_act;
    }

    public sym_vals fn_val(LinkedList<sym_vals> args){
        System.out.println("Doing android newline");

        BlockingOnUIRunnable actionRunnable = new BlockingOnUIRunnable( act, new BlockingOnUIRunnableListener()
        {
            public void onRunOnUIThread()
            {
                // Execute your activity code here
                runText.setText(
                        runText.getText().toString() + "\n");
                ((EditText)runText).setSelection(runText.getText().length());
            }
        } );

        actionRunnable.startOnUiAndWait();


        return new sym_num(0);
    }
    FullscreenActivity act;
    TextView runText;
}
