/* Copyright 2015 Thomas Flynn */
/*This file is part of asc.

    asc is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    asc is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with asc.  If not, see <http://www.gnu.org/licenses/>.*/

package com.androidscheme.tom.myapplication;

import android.content.IntentSender;
import android.os.Bundle;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesUtil;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.common.api.ResultCallback;
import com.google.android.gms.drive.Drive;
import com.google.android.gms.drive.DriveApi;
import com.google.android.gms.drive.DriveFile;
import com.google.android.gms.drive.DriveResource;
import com.google.android.gms.drive.MetadataChangeSet;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;

/**
 * Created by tom on 8/12/15.
 */
public class driveio implements
        GoogleApiClient.OnConnectionFailedListener {
    static final int MODE_OPEN=0;
    static final int MODE_SAVE=1;
    static final int MODE_NONE=-1;

    static final int REQUEST_CODE_RESOLUTION = 1;
    static final int REQUEST_CODE_CREATOR = 1;

    driveio(FullscreenActivity _act){
        act=_act;
        mGoogleApiClient = new GoogleApiClient.Builder(act)
                .addApi(Drive.API)
                .addScope(Drive.SCOPE_FILE)
                .addConnectionCallbacks(act)
                .addOnConnectionFailedListener(this)
                .build();

    }

    @Override
    public void onConnectionFailed(ConnectionResult connectionResult) {
        System.out.println("In connectionfailed");
        if (connectionResult.hasResolution()) {
            System.out.println("DOing resolution");
            try {
                connectionResult.startResolutionForResult(act, driveio.REQUEST_CODE_RESOLUTION);
            } catch (IntentSender.SendIntentException e) {
                // Unable to resolve, message user appropriately
            }
        } else {
            System.out.println("SHOWING error dialog");
            GooglePlayServicesUtil.getErrorDialog(connectionResult.getErrorCode(), act, 0).show();
        }
    }

    /* ---------- LOAD FNS ------------- */

    void start_load(){ //called when user clicked "Open from Google Drive"
        mode=MODE_OPEN;
        mGoogleApiClient.registerConnectionCallbacks(connect_then_load);
        if(!mGoogleApiClient.isConnected()){
            mGoogleApiClient.connect();
        }
    }

    final GoogleApiClient.ConnectionCallbacks connect_then_load = //called after connection setup
            new GoogleApiClient.ConnectionCallbacks() {
                @Override
                public void onConnected(Bundle bundle) {
                   /* Query query = new Query.Builder()
                            .addFilter(Filters.eq(SearchableField.MIME_TYPE, "text/plain"))
                            .build();*/
                    String[] types = new String[1];
                    types[0] = "text/plain";
                    /*Drive.DriveApi.query(mGoogleApiClient, query)
                            .setResultCallback(metadataCallback);*/
                    IntentSender isend= Drive.DriveApi.newOpenFileActivityBuilder()
                            .setActivityTitle("Load")
                            .setMimeType(types)
                            .build(mGoogleApiClient);
                    try {
                        act.startIntentSenderForResult(isend,
                                REQUEST_CODE_CREATOR,
                                null,0, 0, 0);
                    } catch (IntentSender.SendIntentException e) {
                    }
                }

                @Override
                public void onConnectionSuspended(int i) {

                }
            };

    ResultCallback<DriveApi.DriveContentsResult> contentcb = //called after user selects a file
            new ResultCallback<DriveApi.DriveContentsResult>() {
                @Override
                public void onResult(DriveApi.DriveContentsResult result) {
                    newprog = act.string_from_stream
                            (result.getDriveContents().getInputStream());
                    System.out.println("Got some data");

                    //DriveFile googleDriveFile  = Drive.DriveApi.getFile(mGoogleApiClient,
                     //       result.getDriveContents().getDriveId());

                   // googleDriveFile.getMetadata(mGoogleApiClient).setResultCallback(namecb);


                }
            };

    ResultCallback<DriveResource.MetadataResult> namecb = //once they select the file, here comes the name
            new ResultCallback<DriveResource.MetadataResult>() {
                @Override
                public void onResult(DriveResource.MetadataResult metadataResult) {
                    act.newsnippet(metadataResult.getMetadata().getTitle(), //finally, add new snippet
                            newprog);
                    mGoogleApiClient.unregisterConnectionCallbacks(connect_then_load);
                    mode=MODE_NONE;
                }
            };



    /* ------------ BEGIN SAVE FNS ----------- */

    void start_save(){ //called when user selects "Save to Google Drive"
        mode=MODE_SAVE;
        mGoogleApiClient.registerConnectionCallbacks(connect_then_save);
        if (!mGoogleApiClient.isConnected()) {
            mGoogleApiClient.connect();
        }
    }

    GoogleApiClient.ConnectionCallbacks connect_then_save = //called after connection setup
            new GoogleApiClient.ConnectionCallbacks() {
                @Override
                public void onConnected(Bundle bundle) {
                    Drive.DriveApi.newDriveContents(mGoogleApiClient)
                            .setResultCallback(savecb);
                }

                @Override
                public void onConnectionSuspended(int i) {

                }
            };

    ResultCallback<DriveApi.DriveContentsResult> savecb = //write the data.
            new ResultCallback<DriveApi.DriveContentsResult>() {
                @Override
                public void onResult(DriveApi.DriveContentsResult result) {

                    if (!result.getStatus().isSuccess()) {
                        // Handle error
                        return;
                    }
                    OutputStream outputStream = result.getDriveContents().getOutputStream();
                    Writer writer = new OutputStreamWriter(outputStream);
                    try {
                        writer.write(act.get_current_source());
                        writer.close();
                    } catch (IOException e) {
                        System.out.println("Mac.");
                    }
                    MetadataChangeSet metadataChangeSet = new MetadataChangeSet.Builder()
                            .setMimeType("text/plain")
                            .setTitle(act.get_current_title())
                            .build();
                    IntentSender intentSender = Drive.DriveApi
                            .newCreateFileActivityBuilder()
                            .setInitialMetadata(metadataChangeSet)
                            .setInitialDriveContents(result.getDriveContents())
                            .build(mGoogleApiClient);
                    mGoogleApiClient.unregisterConnectionCallbacks(connect_then_save);
                    try {
                        act.startIntentSenderForResult(intentSender, REQUEST_CODE_CREATOR, null, 0, 0, 0);
                    } catch (IntentSender.SendIntentException e) {
                        // Handle the exception
                    }
                }
            };


    GoogleApiClient mGoogleApiClient;

    int mode;
    String newprog;
    FullscreenActivity act;
}
